/* global HTMLSelectElement, L */

import Icon from './Icon.js';
import Settings from './Settings.js';

const cachedData = {};
let marker;
let poi;
let poiAlreadySetToCenter = false;
const markers = [];

const initLocrMap = async () => {
	const locrMapElement = document.querySelector( 'locr-map' );
	if ( ! locrMapElement ) {
		return;
	}

	locrMapElement.addEventListener(
		'custom-element-initialized',
		async function () {
			locrMapElement.addEventListener(
				'set-print-format',
				function ( eventSetPrintFormat ) {
					if ( typeof wp !== 'undefined' && wp.hooks ) {
						wp.hooks.doAction( 'locrMAPSSetPrintFormat', {
							printFormat: eventSetPrintFormat.detail.printFormat,
						} );
					}
				}
			);

			document.addEventListener(
				'locr.locrmaps.fit-bounds',
				function ( eventSetBounds ) {
					locrMapElement.fitBounds( [
						[
							eventSetBounds.detail.bounds.south,
							eventSetBounds.detail.bounds.west,
						],
						[
							eventSetBounds.detail.bounds.north,
							eventSetBounds.detail.bounds.east,
						],
					] );

					if ( eventSetBounds.detail.updateIconPosition && marker ) {
						marker.setLatLng( [
							( eventSetBounds.detail.bounds.south +
								eventSetBounds.detail.bounds.north ) /
								2,
							( eventSetBounds.detail.bounds.west +
								eventSetBounds.detail.bounds.east ) /
								2,
						] );
					}
				}
			);
			document.addEventListener(
				'locr.locrmaps.set-center',
				function ( eventSetCenter ) {
					locrMapElement.setView(
						window.L.latLng(
							eventSetCenter.detail.center.latitude,
							eventSetCenter.detail.center.longitude
						),
						eventSetCenter.detail.zoom || locrMapElement.getZoom()
					);

					if ( eventSetCenter.detail.updateIconPosition && marker ) {
						marker.setLatLng( [
							eventSetCenter.detail.center.latitude,
							eventSetCenter.detail.center.longitude,
						] );
					}
				}
			);
			document.addEventListener(
				'locr.locrmaps.set-overlay-template',
				function ( eventSetOverlayTemplate ) {
					if ( eventSetOverlayTemplate.detail.overlayTemplateData ) {
						locrMapElement.setMapOverlayByData(
							eventSetOverlayTemplate.detail.overlayTemplateData
						);
					}
				}
			);
			document.addEventListener(
				'locr.locrmaps.set-text-overlay',
				function ( eventSetTextOverlay ) {
					if (
						typeof eventSetTextOverlay.detail.id === 'string' &&
						typeof eventSetTextOverlay.detail.content === 'string'
					) {
						locrMapElement.replaceTextSvgLayerContent(
							eventSetTextOverlay.detail.id,
							eventSetTextOverlay.detail.content
						);
					}
				}
			);
			document.addEventListener(
				'locr.locrmaps.set-icon',
				async function ( eventSetIcon ) {
					const icon = await getIconByName(
						eventSetIcon.detail.icon
					);
					if (
						icon &&
						typeof eventSetIcon.detail.draggable === 'boolean'
					) {
						icon.draggable = eventSetIcon.detail.draggable;
					}

					let latitude;
					let longitude;
					if ( marker ) {
						const markerLatLng = marker.getLatLng();
						if ( markerLatLng ) {
							latitude = markerLatLng.lat;
							longitude = markerLatLng.lng;
						}
						marker.remove();
					}

					if ( icon ) {
						if ( isNaN( latitude ) || isNaN( longitude ) ) {
							const center = locrMapElement.getCenter();
							if ( center ) {
								latitude = center.lat;
								longitude = center.lng;
							}
						}

						marker = window.L.marker(
							window.L.latLng( latitude, longitude ),
							{
								icon: icon.toLeafletIcon(),
								draggable: icon.draggable,
							}
						);
						locrMapElement.addMarker( marker );
					}
				}
			);
			document.addEventListener(
				'locr.locrmaps.set-poi',
				async function ( eventSetIcon ) {
					const icon = await getIconByName(
						eventSetIcon.detail.icon
					);
					if (
						icon &&
						typeof eventSetIcon.detail.draggable === 'boolean'
					) {
						icon.draggable = eventSetIcon.detail.draggable;
					}

					let latitude;
					let longitude;
					if (
						poi &&
						typeof poi.latitude === 'number' &&
						typeof poi.longitude === 'number'
					) {
						latitude = poi.latitude;
						longitude = poi.longitude;
					}

					if ( icon ) {
						poi = {
							iconData: { ...icon.toLeafletOptions() },
							id: 'my-poi',
						};
						if (
							( isNaN( latitude ) || isNaN( longitude ) ) &&
							! poiAlreadySetToCenter
						) {
							const center = locrMapElement.getCenter();
							if ( center ) {
								poi.latitude = center.lat;
								poi.longitude = center.lng;
								poiAlreadySetToCenter = true;
							}
						}
						locrMapElement.addOrReplacePoiToMapOverlay( poi );
					}
				}
			);
			document.addEventListener(
				'locr.locrmaps.update-icon-position',
				function ( eventUpdateIconPosition ) {
					if ( ! eventUpdateIconPosition.detail.center ) {
						return;
					}

					if ( marker ) {
						marker.setLatLng( [
							eventUpdateIconPosition.detail.center.latitude,
							eventUpdateIconPosition.detail.center.longitude,
						] );
					} else if ( poi ) {
						poi = {
							iconData: { ...poi.iconData },
							id: poi.id,
							latitude:
								eventUpdateIconPosition.detail.center.latitude,
							longitude:
								eventUpdateIconPosition.detail.center.longitude,
						};
						locrMapElement.addOrReplacePoiToMapOverlay( poi );
					}
				}
			);
			document.addEventListener(
				'locr.locrmaps.set-print-format',
				function ( eventSetPrintFormat ) {
					locrMapElement.setPrintFormat(
						eventSetPrintFormat.detail.printFormatString
					);
				}
			);
			document.addEventListener(
				'locr.locrmaps.set-style',
				async function ( eventSetStyle ) {
					locrMapElement.setStyle( eventSetStyle.detail.style );
				}
			);
			locrMapElement.on( 'load', () => {
				if ( typeof wp !== 'undefined' && wp.hooks ) {
					wp.hooks.doAction( 'locrMAPSInitialized' );
				}
			} );

			const settings = await Settings.init();

			const locrmapsStylesElement =
				document.getElementById( 'locrmaps-styles' );
			if ( locrmapsStylesElement instanceof HTMLSelectElement ) {
				const previousStyle = locrmapsStylesElement.value;
				locrmapsStylesElement.value = settings.style;
				if ( locrmapsStylesElement.value === '' ) {
					locrmapsStylesElement.selectedIndex = 0;
				}
				if ( previousStyle !== locrmapsStylesElement.value ) {
					locrmapsStylesElement.dispatchEvent(
						new Event( 'change' )
					);
				}
			}

			let bounds;
			let center = L.latLng(
				settings.position.latitude,
				settings.position.longitude
			);
			if ( settings.boundings ) {
				bounds = L.latLngBounds(
					L.latLng(
						settings.boundings[ 1 ],
						settings.boundings[ 0 ]
					),
					L.latLng( settings.boundings[ 3 ], settings.boundings[ 2 ] )
				);
			}
			if ( settings.locations.length > 0 ) {
				for ( const location of settings.locations ) {
					const icon = await getIconById( location.icon_id );
					if ( icon ) {
						addIconOnPosition(
							locrMapElement,
							icon,
							location.position
						);
					}
				}
				const minMaxCoordinates = settings.locations.reduce(
					function ( accumulator, currentValue ) {
						return {
							minLat: Math.min(
								accumulator.minLat,
								currentValue.position.latitude
							),
							maxLat: Math.max(
								accumulator.maxLat,
								currentValue.position.latitude
							),
							minLng: Math.min(
								accumulator.minLng,
								currentValue.position.longitude
							),
							maxLng: Math.max(
								accumulator.maxLng,
								currentValue.position.longitude
							),
						};
					},
					{
						minLat: 90,
						maxLat: -90,
						minLng: 180,
						maxLng: -180,
					}
				);

				if ( settings.locations.length === 1 ) {
					center = L.latLng(
						minMaxCoordinates.minLat,
						minMaxCoordinates.minLng
					);
				} else {
					center = L.latLng(
						( minMaxCoordinates.minLat +
							minMaxCoordinates.maxLat ) /
							2,
						( minMaxCoordinates.minLng +
							minMaxCoordinates.maxLng ) /
							2
					);
					bounds = L.latLngBounds(
						L.latLng(
							minMaxCoordinates.minLat,
							minMaxCoordinates.minLng
						),
						L.latLng(
							minMaxCoordinates.maxLat,
							minMaxCoordinates.maxLng
						)
					);
				}
			}

			if ( bounds ) {
				locrMapElement.fitBounds( bounds, { padding: [ 25, 25 ] } );
				if ( typeof wp !== 'undefined' && wp.hooks ) {
					wp.hooks.doAction( 'locrMAPSInitialFitBounds' );
				}
			} else {
				locrMapElement.setView( center, settings.zoom );
				if ( typeof wp !== 'undefined' && wp.hooks ) {
					wp.hooks.doAction( 'locrMAPSInitialSetView' );
				}
			}
		}
	);
};

document.addEventListener( 'DOMContentLoaded', initLocrMap );

/**
 * @param {Element}                               locrMap
 * @param {Icon}                                  icon
 * @param {{latitude: number, longitude: number}} position
 */
function addIconOnPosition( locrMap, icon, position ) {
	const leafletMarker = window.L.marker(
		window.L.latLng( position.latitude, position.longitude ),
		{
			icon: icon.toLeafletIcon(),
		}
	);
	locrMap.addMarker( leafletMarker );

	markers.push( leafletMarker );
}

/**
 * @param {number} iconId
 * @return {Promise<Icon|null>} Icon
 */
async function getIconById( iconId ) {
	const apiFetchHandler = window.wp?.apiFetch || window.wp?.apiRequest;
	if ( ! apiFetchHandler ) {
		window.console.warn( 'API Fetch handler not available.' );
		return null;
	}

	try {
		if ( ! cachedData.icons ) {
			const allIconData = await apiFetchHandler( {
				path: `/locrmaps/icons/all`,
			} );
			if ( allIconData.icons ) {
				cachedData.icons = allIconData.icons;
			}
		}
		if ( ! cachedData.icons ) {
			return this;
		}

		for ( const iconName in cachedData.icons ) {
			if ( cachedData.icons[ iconName ].id === iconId ) {
				return new Icon( {
					name: iconName,
					...cachedData.icons[ iconName ],
				} );
			}
		}
	} catch ( exc ) {
		// window.alert( 'failed to fetch icon data.' );
	}

	return null;
}

/**
 * @param {string} iconName
 * @return {Promise<Icon|null>} Icon
 */
async function getIconByName( iconName ) {
	const apiFetchHandler = window.wp?.apiFetch || window.wp?.apiRequest;
	if ( ! apiFetchHandler ) {
		window.console.warn( 'API Fetch handler not available.' );
		return null;
	}

	try {
		if ( ! cachedData.icons ) {
			const allIconData = await apiFetchHandler( {
				path: `/locrmaps/icons/all`,
			} );
			if ( allIconData.icons ) {
				cachedData.icons = allIconData.icons;
			}
		}
		if ( ! cachedData.icons ) {
			window.console.warn( 'if ( ! cachedData.icons ) => return null;' );
			return null;
		}

		if ( cachedData.icons[ iconName ] ) {
			return new Icon( {
				name: iconName,
				...cachedData.icons[ iconName ],
			} );
		}
	} catch ( exc ) {
		// window.alert( 'failed to fetch icon data.' );
	}

	return null;
}

export { initLocrMap };
